@@ -60,6 +60,8 @@ private Q_SLOTS:
     void suspendFileCopy();
     void suspendCopy();
     void listRecursive();
+    void listRecursiveExcludeDots();
+    void listRecursiveExcludeDotsHiddenToggle();
     void multipleListRecursive();
     void listFile();
     void killJob();
