@@ -48,6 +48,11 @@ static const Qt::CaseSensitivity cs = Qt::CaseSensitive;
 // Linux
 #if HAVE_LIB_MOUNT
 #include <libmount/libmount.h>
+#include <sys/sysmacros.h>
+#if HAVE_STATX_MNT_ID
+#include <fcntl.h>
+#include <sys/stat.h>
+#endif
 #endif
 
 static bool isNetfs(const QString &mountType)
@@ -77,7 +82,7 @@ class KMountPointPrivate
 public:
     void resolveGvfsMountPoints(KMountPoint::List &result);
     void finalizePossibleMountPoint(KMountPoint::DetailsNeededFlags infoNeeded);
-    void finalizeCurrentMountPoint(KMountPoint::DetailsNeededFlags infoNeeded);
+    void setAdditionalDetails(KMountPoint::DetailsNeededFlags infoNeeded);
 
     QString m_mountedFrom;
     QString m_device; // Only available when the NeedRealDeviceName flag was set.
@@ -85,6 +90,10 @@ public:
     QString m_mountType;
     QStringList m_mountOptions;
     dev_t m_deviceId = 0;
+#if HAVE_STATX_MNT_ID
+    // the mount id seems to start at 1. 0 should be safe as a undefined default
+    __u64 m_mountId = 0;
+#endif
     bool m_isNetFs = false;
 };
 
@@ -100,9 +109,9 @@ KMountPoint::~KMountPoint() = default;
 #ifdef MNTOPT_NAMES
 static struct mntoptnames bsdOptionNames[] = {MNTOPT_NAMES};
 
-/** @brief Get mount options from @p flags and puts human-readable version in @p list
+/* Get mount options from flags and puts human-readable version in list
  *
- * Appends all positive options found in @p flags to the @p list
+ * Appends all positive options found in flags to the list
  * This is roughly paraphrased from FreeBSD's mount.c, prmount().
  */
 static void translateMountOptions(QStringList &list, uint64_t flags)
@@ -121,9 +130,9 @@ static void translateMountOptions(QStringList &list, uint64_t flags)
     }
 }
 #else
-/** @brief Get mount options from @p flags and puts human-readable version in @p list
+/* Get mount options from flags and puts human-readable version in list
  *
- * This default version just puts the hex representation of @p flags
+ * This default version just puts the hex representation of flags
  * in the list, because there is no human-readable version.
  */
 static void translateMountOptions(QStringList &list, uint64_t flags)
@@ -147,17 +156,13 @@ void KMountPointPrivate::finalizePossibleMountPoint(KMountPoint::DetailsNeededFl
         m_mountedFrom = potentialDevice;
     }
 
-    if (infoNeeded & KMountPoint::NeedRealDeviceName) {
-        if (m_mountedFrom.startsWith(QLatin1Char('/'))) {
-            m_device = QFileInfo(m_mountedFrom).canonicalFilePath();
-        }
-    }
+    setAdditionalDetails(infoNeeded);
 
     // Chop trailing slash
     Utils::removeTrailingSlash(m_mountedFrom);
 }
 
-void KMountPointPrivate::finalizeCurrentMountPoint(KMountPoint::DetailsNeededFlags infoNeeded)
+void KMountPointPrivate::setAdditionalDetails(KMountPoint::DetailsNeededFlags infoNeeded)
 {
     if (infoNeeded & KMountPoint::NeedRealDeviceName) {
         if (m_mountedFrom.startsWith(QLatin1Char('/'))) {
@@ -318,7 +323,7 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
             }
         }
 
-        mp->d->finalizeCurrentMountPoint(infoNeeded);
+        mp->d->setAdditionalDetails(infoNeeded);
         // TODO: Strip trailing '/' ?
         result.append(mp);
     }
@@ -359,21 +364,25 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
                 mp->d->m_mountPoint = QFile::decodeName(mnt_fs_get_target(fs));
                 mp->d->m_mountType = QFile::decodeName(mnt_fs_get_fstype(fs));
                 mp->d->m_isNetFs = mnt_fs_is_netfs(fs) == 1;
-                mp->d->m_deviceId = mnt_fs_get_devno(fs);
+#if HAVE_STATX_MNT_ID
+                // mnt_fs_get_id actually returns a int whereas the kernel uses __u64
+                int mnt_id = mnt_fs_get_id(fs);
+                if (mnt_id != -1) {
+                    mp->d->m_mountId = (__u64)mnt_id;
+                }
+#endif
+
+                if (struct statx buff; statx(AT_FDCWD, mnt_fs_get_target(fs), AT_STATX_DONT_SYNC, STATX_INO, &buff) == 0) {
+                    mp->d->m_deviceId = makedev(buff.stx_dev_major, buff.stx_dev_minor);
+                }
 
                 if (infoNeeded & NeedMountOptions) {
                     mp->d->m_mountOptions = QFile::decodeName(mnt_fs_get_options(fs)).split(QLatin1Char(','));
                 }
 
-                if (infoNeeded & NeedRealDeviceName) {
-                    if (mp->d->m_mountedFrom.startsWith(QLatin1Char('/'))) {
-                        mp->d->m_device = mp->d->m_mountedFrom;
-                    }
-                }
-
                 mp->d->resolveGvfsMountPoints(result);
 
-                mp->d->finalizeCurrentMountPoint(infoNeeded);
+                mp->d->setAdditionalDetails(infoNeeded);
                 result.push_back(mp);
             }
 
@@ -435,10 +444,23 @@ KMountPoint::Ptr KMountPoint::List::findByPath(const QString &path) const
     /* If the path contains symlinks, get the real name */
     QFileInfo fileinfo(path);
     // canonicalFilePath won't work unless file exists
-    const QString realPath = fileinfo.exists() ? fileinfo.canonicalFilePath() : fileinfo.absolutePath();
+    const QString realPath = fileinfo.exists() && !fileinfo.isSymLink() ? fileinfo.canonicalFilePath() : fileinfo.absolutePath();
 #endif
 
     KMountPoint::Ptr result;
+#if HAVE_STATX_MNT_ID
+    // If we have statx, there is no need to guess. take the mount id from statx and get the mountpoint.
+    if (struct statx buff; statx(0, QFile::encodeName(realPath).constData(), AT_NO_AUTOMOUNT, STATX_MNT_ID, &buff) == 0) {
+        auto it = std::find_if(this->cbegin(), this->cend(), [&buff](const KMountPoint::Ptr &mountPtr) {
+            return mountPtr->d->m_mountId == buff.stx_mnt_id;
+        });
+
+        if (it != this->cend()) {
+            return *it;
+        }
+    }
+#endif
+
 
     if (QT_STATBUF buff; QT_LSTAT(QFile::encodeName(realPath).constData(), &buff) == 0) {
         auto it = std::find_if(this->cbegin(), this->cend(), [&buff, &realPath](const KMountPoint::Ptr &mountPtr) {
