@@ -20,31 +20,32 @@ using namespace KIO;
 class KIO::ListJobPrivate : public KIO::SimpleJobPrivate
 {
 public:
-    ListJobPrivate(const QUrl &url, bool _recursive, const QString &prefix, const QString &displayPrefix, bool _includeHidden)
+    ListJobPrivate(const QUrl &url, bool _recursive, const QString &prefix, const QString &displayPrefix, ListJob::ListFlags listFlags)
         : SimpleJobPrivate(url, CMD_LISTDIR, QByteArray())
         , recursive(_recursive)
-        , includeHidden(_includeHidden)
+        , listFlags(listFlags)
         , m_prefix(prefix)
         , m_displayPrefix(displayPrefix)
         , m_processedEntries(0)
     {
     }
     bool recursive;
-    bool includeHidden;
+    ListJob::ListFlags listFlags;
     QString m_prefix;
     QString m_displayPrefix;
     unsigned long m_processedEntries;
     QUrl m_redirectionURL;
 
-    /**
-     * @internal
+    /*!
+     * \internal
      * Called by the scheduler when a @p worker gets to
      * work on this job.
-     * @param worker the worker that starts working on this job
+     * \a worker the worker that starts working on this job
      */
     void start(Worker *worker) override;
 
-    void slotListEntries(const KIO::UDSEntryList &list);
+    void filterAndEmitEntries(const KIO::UDSEntryList &list);
+    void maybeRecurse(const KIO::UDSEntryList &list);
     void slotRedirection(const QUrl &url);
     void gotEntries(KIO::Job *subjob, const KIO::UDSEntryList &list);
     void slotSubError(ListJob *job, ListJob *subJob);
@@ -52,18 +53,18 @@ public:
     Q_DECLARE_PUBLIC(ListJob)
 
     static inline ListJob *
-    newJob(const QUrl &u, bool _recursive, const QString &prefix, const QString &displayPrefix, bool _includeHidden, JobFlags flags = HideProgressInfo)
+    newJob(const QUrl &u, bool _recursive, const QString &prefix, const QString &displayPrefix, ListJob::ListFlags listFlags, JobFlags flags = HideProgressInfo)
     {
-        ListJob *job = new ListJob(*new ListJobPrivate(u, _recursive, prefix, displayPrefix, _includeHidden));
+        ListJob *job = new ListJob(*new ListJobPrivate(u, _recursive, prefix, displayPrefix, listFlags));
         job->setUiDelegate(KIO::createDefaultJobUiDelegate());
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
         return job;
     }
-    static inline ListJob *newJobNoUi(const QUrl &u, bool _recursive, const QString &prefix, const QString &displayPrefix, bool _includeHidden)
+    static inline ListJob *newJobNoUi(const QUrl &u, bool _recursive, const QString &prefix, const QString &displayPrefix, ListJob::ListFlags listFlags)
     {
-        return new ListJob(*new ListJobPrivate(u, _recursive, prefix, displayPrefix, _includeHidden));
+        return new ListJob(*new ListJobPrivate(u, _recursive, prefix, displayPrefix, listFlags));
     }
 };
 
@@ -81,74 +82,104 @@ ListJob::~ListJob()
 {
 }
 
-void ListJobPrivate::slotListEntries(const KIO::UDSEntryList &list)
+void ListJobPrivate::maybeRecurse(const KIO::UDSEntryList &list)
 {
     Q_Q(ListJob);
+
+    const bool includeHidden = listFlags.testFlag(ListJob::ListFlag::IncludeHidden);
+
+    if (!recursive) {
+        return;
+    }
+
+    for (const KIO::UDSEntry &entry : list) {
+        QUrl itemURL;
+        const QString udsUrl = entry.stringValue(KIO::UDSEntry::UDS_URL);
+        QString filename;
+        if (!udsUrl.isEmpty()) {
+            itemURL = QUrl(udsUrl);
+            filename = itemURL.fileName();
+        } else { // no URL, use the name
+            itemURL = q->url();
+            filename = entry.stringValue(KIO::UDSEntry::UDS_NAME);
+            Q_ASSERT(!filename.isEmpty()); // we'll recurse forever otherwise :)
+            itemURL.setPath(Utils::concatPaths(itemURL.path(), filename));
+        }
+
+        if (!entry.isDir() || entry.isLink()) {
+            continue;
+        }
+
+        Q_ASSERT(!filename.isEmpty());
+        QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
+        if (displayName.isEmpty()) {
+            displayName = filename;
+        }
+        // skip hidden dirs when listing if requested
+        if (filename != QLatin1String("..") && filename != QLatin1String(".") && (includeHidden || filename[0] != QLatin1Char('.'))) {
+            ListJob *job = ListJobPrivate::newJobNoUi(itemURL,
+                                                      true /*recursive*/,
+                                                      m_prefix + filename + QLatin1Char('/'),
+                                                      m_displayPrefix + displayName + QLatin1Char('/'),
+                                                      listFlags);
+            QObject::connect(job, &ListJob::entries, q, [this](KIO::Job *job, const KIO::UDSEntryList &list) {
+                gotEntries(job, list);
+            });
+            QObject::connect(job, &ListJob::subError, q, [this](KIO::ListJob *job, KIO::ListJob *ljob) {
+                slotSubError(job, ljob);
+            });
+
+            q->addSubjob(job);
+        }
+    }
+}
+
+void ListJobPrivate::filterAndEmitEntries(const KIO::UDSEntryList &list)
+{
+    Q_Q(ListJob);
+
+    const bool includeHidden = listFlags.testFlag(ListJob::ListFlag::IncludeHidden);
+    const bool excludeDot = listFlags.testFlag(ListJob::ListFlag::ExcludeDot);
+    const bool excludeDotDot = listFlags.testFlag(ListJob::ListFlag::ExcludeDotDot);
+
     // Emit progress info (takes care of emit processedSize and percent)
     m_processedEntries += list.count();
     slotProcessedSize(m_processedEntries);
 
-    if (recursive) {
-        UDSEntryList::ConstIterator it = list.begin();
-        const UDSEntryList::ConstIterator end = list.end();
-
-        for (; it != end; ++it) {
-            const UDSEntry &entry = *it;
-
-            QUrl itemURL;
-            const QString udsUrl = entry.stringValue(KIO::UDSEntry::UDS_URL);
-            QString filename;
-            if (!udsUrl.isEmpty()) {
-                itemURL = QUrl(udsUrl);
-                filename = itemURL.fileName();
-            } else { // no URL, use the name
-                itemURL = q->url();
-                filename = entry.stringValue(KIO::UDSEntry::UDS_NAME);
-                Q_ASSERT(!filename.isEmpty()); // we'll recurse forever otherwise :)
-                itemURL.setPath(Utils::concatPaths(itemURL.path(), filename));
-            }
-
-            if (entry.isDir() && !entry.isLink()) {
-                Q_ASSERT(!filename.isEmpty());
-                QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
-                if (displayName.isEmpty()) {
-                    displayName = filename;
-                }
-                // skip hidden dirs when listing if requested
-                if (filename != QLatin1String("..") && filename != QLatin1String(".") && (includeHidden || filename[0] != QLatin1Char('.'))) {
-                    ListJob *job = ListJobPrivate::newJobNoUi(itemURL,
-                                                              true /*recursive*/,
-                                                              m_prefix + filename + QLatin1Char('/'),
-                                                              m_displayPrefix + displayName + QLatin1Char('/'),
-                                                              includeHidden);
-                    QObject::connect(job, &ListJob::entries, q, [this](KIO::Job *job, const KIO::UDSEntryList &list) {
-                        gotEntries(job, list);
-                    });
-                    QObject::connect(job, &ListJob::subError, q, [this](KIO::ListJob *job, KIO::ListJob *ljob) {
-                        slotSubError(job, ljob);
-                    });
-
-                    q->addSubjob(job);
-                }
-            }
-        }
-    }
-
     // Not recursive, or top-level of recursive listing : return now (send . and .. as well)
     // exclusion of hidden files also requires the full sweep, but the case for full-listing
     // a single dir is probably common enough to justify the shortcut
-    if (m_prefix.isNull() && includeHidden) {
+    if (m_prefix.isNull() && includeHidden && !excludeDot && !excludeDotDot) {
         Q_EMIT q->entries(q, list);
     } else {
         UDSEntryList newlist = list;
 
-        auto removeFunc = [this](const UDSEntry &entry) {
+        // Mind that the repeated iteration hardly makes a performance difference here, presumably
+        // because the compiler is smart enough to optimize it away.
+        // Same for repeat calls to stringValue().
+        // Benchmark before you decide to make improvements here!
+
+        auto removeFunc = [this, includeHidden, excludeDot, excludeDotDot](const UDSEntry &entry) {
             const QString filename = entry.stringValue(KIO::UDSEntry::UDS_NAME);
+
             // Avoid returning entries like subdir/. and subdir/.., but include . and .. for
-            // the toplevel dir, and skip hidden files/dirs if that was requested
-            const bool shouldEmit = (m_prefix.isNull() || (filename != QLatin1String("..") && filename != QLatin1String(".")))
-                && (includeHidden || (filename[0] != QLatin1Char('.')));
-            return !shouldEmit;
+            // the toplevel dir unless excludeDot/excludeDotDot were passed
+            if (m_prefix.isNull()) {
+                if ((excludeDotDot && filename == QLatin1String("..")) || (excludeDot && filename == QLatin1String("."))) {
+                    return true;
+                }
+            } else {
+                if (filename == QLatin1String("..") || filename == QLatin1String(".")) {
+                    return true;
+                }
+            }
+
+            // skip hidden files/dirs if that was requested
+            if (!includeHidden && filename[0] == QLatin1Char('.')) {
+                return true;
+            }
+
+            return false;
         };
         newlist.erase(std::remove_if(newlist.begin(), newlist.end(), removeFunc), newlist.end());
 
@@ -160,7 +191,6 @@ void ListJobPrivate::slotListEntries(const KIO::UDSEntryList &list)
                 displayName = filename;
             }
 
-            // ## Didn't find a way to use the iterator instead of re-doing a key lookup
             newone.replace(KIO::UDSEntry::UDS_NAME, m_prefix + filename);
             newone.replace(KIO::UDSEntry::UDS_DISPLAY_NAME, m_displayPrefix + displayName);
         }
@@ -233,14 +263,14 @@ void ListJob::slotFinished()
     SimpleJob::slotFinished();
 }
 
-ListJob *KIO::listDir(const QUrl &url, JobFlags flags, bool includeHidden)
+ListJob *KIO::listDir(const QUrl &url, JobFlags flags, ListJob::ListFlags listFlags)
 {
-    return ListJobPrivate::newJob(url, false, QString(), QString(), includeHidden, flags);
+    return ListJobPrivate::newJob(url, false, QString(), QString(), listFlags, flags);
 }
 
-ListJob *KIO::listRecursive(const QUrl &url, JobFlags flags, bool includeHidden)
+ListJob *KIO::listRecursive(const QUrl &url, JobFlags flags, ListJob::ListFlags listFlags)
 {
-    return ListJobPrivate::newJob(url, true, QString(), QString(), includeHidden, flags);
+    return ListJobPrivate::newJob(url, true, QString(), QString(), listFlags, flags);
 }
 
 void ListJob::setUnrestricted(bool unrestricted)
@@ -262,8 +292,9 @@ void ListJobPrivate::start(Worker *worker)
         QTimer::singleShot(0, q, &ListJob::slotFinished);
         return;
     }
-    QObject::connect(worker, &Worker::listEntries, q, [this](const KIO::UDSEntryList &list) {
-        slotListEntries(list);
+    QObject::connect(worker, &Worker::listEntries, q, [this](KIO::UDSEntryList list) {
+        maybeRecurse(list);
+        filterAndEmitEntries(list);
     });
 
     QObject::connect(worker, &Worker::totalSize, q, [this](KIO::filesize_t size) {
