@@ -218,12 +218,6 @@ void KUrlComboBox::setUrls(const QStringList &_urls, OverLoadResolving remove)
             u.setUrl(*it);
         }
 
-        // Don't restore if file doesn't exist anymore
-        if (u.isLocalFile() && !QFile::exists(u.toLocalFile())) {
-            ++it;
-            continue;
-        }
-
         std::unique_ptr<KUrlComboBoxPrivate::KUrlComboItem> item(new KUrlComboBoxPrivate::KUrlComboItem(u, d->getIcon(u)));
         d->insertUrlItem(item.get());
         d->itemList.push_back(std::move(item));
@@ -414,6 +408,12 @@ void KUrlComboBox::mouseMoveEvent(QMouseEvent *event)
     KComboBox::mouseMoveEvent(event);
 }
 
+void KUrlComboBox::focusOutEvent(QFocusEvent *event)
+{
+    Q_EMIT onFocusOut();
+    KComboBox::focusOutEvent(event);
+}
+
 QIcon KUrlComboBoxPrivate::getIcon(const QUrl &url) const
 {
     if (myMode == KUrlComboBox::Directories) {
