@@ -211,9 +211,17 @@ Q_SIGNALS:
      */
     void urlActivated(const QUrl &url);
 
+    /*!
+     * Emitted when combobox loses focus.
+     *
+     * \since 6.23
+     */
+    void onFocusOut();
+
 protected:
     void mousePressEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
+    void focusOutEvent(QFocusEvent *event) override;
 
 private:
     friend class KUrlComboBoxPrivate;
